/* -*- C++ -*- Time-stamp: <10 Mar 01 18:03:20 Michael Bischoff> */

/*****************************************************************************/
/*									     */
/*									     */
/*	QMatchIt version 1.0 -- file fields.cpp				     */
/*									     */
/*	Implementation of the superclasses of the field class of QMatchIt    */
/*	written by Michael Bischoff (Michael.Bischoff@gmx.net)		     */
/*	November-2000							     */
/*	see COPYRIGHT.qmatchit for Copyright details			     */
/*									     */
/*									     */
/*****************************************************************************/

#include "fields.h"

const char f_magnet::asciitab[4] = { 'L', 'M', 'R', 'D' };
const char f_teleporter::asciitab[3] = { 'T', 'Z', 'Y' };
const char f_lift::asciitab[3] = { '!', '_', '&' };

int f_cursor::currentFrame() {
    if (++ctr > 4)
	ctr = 0;
    if (ctr < 3)
	return state ? 26 : 24;
    else
	return 25;
}

int f_unsafe::currentFrame() {
    if (!state) {
	return baseFrame();	// f_unsafe or f_vanisher!
    } else {
	if (ctr >= 6) {
	    state = state_dead;
	    return 30;	// dead
	} else {
	    return baseFrame() + ++ctr;
	}
    }
}

int f_flickers::currentFrame() {
    if (!state) {
	return baseFrame();
    } else {
	if (ctr >= 10) {
	    state = state_dead;
	    return 30;	// dead
	} else {
	    /* explode */
	    if (++ctr & 1)
		return 31;	// black
	    else
		return baseFrame();
	}
    }
}

field *field::newField(int ascii, int x, int y) {
    field *fp = 0;
    int subtype = 0;
    switch (ascii) {
    case RTTI_CURSOR:
	fp = new f_cursor();
	break;
    case RTTI_MAGNET:
	fp = new f_magnet();
	subtype = f_magnet::magnet_up;
	break;
    case 'D':
	fp = new f_magnet();
	subtype = f_magnet::magnet_down;
	break;
    case 'L':
	fp = new f_magnet();
	subtype = f_magnet::magnet_left;
	break;
    case 'R':
	fp = new f_magnet();
	subtype = f_magnet::magnet_right;
	break;
    case RTTI_WALL:
	fp = new f_wall();
	break;
    case RTTI_BOMB:
	fp = new f_bomb();
	break;
    case RTTI_UNSAFE:
	fp = new f_unsafe();
	break;
    case '0':
    case '1':
    case '2':
    case '3':
    case '4':
    case '5':
    case '6':
    case '7':
    case '8':
	fp = new f_box();
	subtype = ascii - '0';
	break;
    case '<':
	fp = new f_oneway();
	subtype = f_oneway::oneway_left;
	break;
    case '>':
	fp = new f_oneway();
	subtype = f_oneway::oneway_right;
	break;
    case RTTI_WEIGHT:
	fp = new f_weight();
	break;
    case RTTI_REMOVABLE:
	fp = new f_removable();
	break;
    case RTTI_INVERTER:
	fp = new f_inverter();
	break;
    case RTTI_VANISHER:
	fp = new f_vanisher();
	break;
    case RTTI_SAVESPOT:
	fp = new f_savespot();
	break;
    case RTTI_TELEPORT:
	fp = new f_teleporter();
	break;
    case 'Z':
	fp = new f_teleporter();
	subtype = 1;
	break;
    case 'Y':
	fp = new f_teleporter();
	subtype = 2;
	break;
    case '^':
	fp = new f_balloon();
	break;
    case '!':
	fp = new f_lift();
	break;
    case '_':
	fp = new f_lift();
	subtype = 1;
	break;
    case '&':
	fp = new f_lift();
	subtype = 2;
	break;
    default:
	return 0;
    }
    fp->init(x, y, subtype);
    return fp;
}
